#include "../webs.h"
#include "loglist.h"

//global var for log list
LOG_LIST_ID		LOG_LIST = NULL;

static NODE_ID Get_Node(void);
static void Remove_Node(void);
static int Is_Empty(void);


/*--------------------------------------------------------------
* ROUTINE NAME - Init_Log_List
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void Init_Log_List(void)
{
	if( LOG_LIST == NULL )
	{
		//alloc memory
		if( !(LOG_LIST = (LOG_LIST_ID)malloc(sizeof(struct _LOG_LIST_))) )
		{
			return;
		}
		
		LOG_LIST->Log_Number = 0;
		LOG_LIST->List_Head  = NULL;
		LOG_LIST->List_Tail  = NULL;
	}
}

/*--------------------------------------------------------------
* ROUTINE NAME - Get_Node
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
static NODE_ID Get_Node(void)
{
	NODE_ID		p;
	
	p = (NODE_ID)malloc(sizeof(struct _Node_));
	
	if( p == NULL )
	{
		return NULL;
	}
	else
	{
		memset(p->DateTime, '0', DATETIME_LEN);
		memset(p->Message,  '0', MESSAGE_LEN);
		memset(p->Source,   '0', SOURCE_LEN);
		memset(p->Note,     '0', NOTE_LEN);
		return p;
	}
}

/*--------------------------------------------------------------
* ROUTINE NAME - Is_Empty
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
static int Is_Empty(void)
{
	return (LOG_LIST->Log_Number == 0) ? 1 : 0;
}

/*--------------------------------------------------------------
* ROUTINE NAME - Insert_Node
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void Insert_Node(char *str_1, char *str_2, char *str_3, char *str_4)
{
	NODE_ID		p;
	
	p = Get_Node();
	if( p == NULL )
	{
		return;
	}
	
	if( Is_Empty() )
	{
		//list is empty, just add a node is ok
		
		LOG_LIST->Log_Number ++;
		LOG_LIST->List_Head = LOG_LIST->List_Tail = p;
	}
	else
	{
		//check max log list number
		if( LOG_LIST->Log_Number >= MAX_LOG_LIST_NUM )
		{
			Remove_Node();
		}

		//list not empty, we add at tail
		LOG_LIST->Log_Number ++;
		LOG_LIST->List_Tail->Next = p;
		LOG_LIST->List_Tail = p;
	}
	
	//copy value to node
	strcpy(p->DateTime, str_1);
	strcpy(p->Message, str_2);
	strcpy(p->Source, str_3);
	strcpy(p->Note, str_4);
	
	p->Next = NULL;
}

/*--------------------------------------------------------------
* ROUTINE NAME - Remove_Node
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
static void Remove_Node(void)
{
	NODE_ID		p;
	
	//we always remove node at head of list
	if( Is_Empty() )
	{
		return;
	}
	else
	{		
		p = LOG_LIST->List_Head;
		
		if( LOG_LIST->Log_Number == 1 )
		{
			LOG_LIST->Log_Number = 0;
			LOG_LIST->List_Head  = NULL;
			LOG_LIST->List_Tail  = NULL;
			free(p);
			return;
		}
		else
		{
			LOG_LIST->Log_Number --;
			LOG_LIST->List_Head = p->Next;
			free(p);
			return;
		}
	}
}

/*--------------------------------------------------------------
* ROUTINE NAME - Free_All_List
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void Free_All_List(void)
{
	while( !Is_Empty() )
	{
		Remove_Node();
	}
}

/*--------------------------------------------------------------
* ROUTINE NAME - Wriet_File
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void Write_File(void)
{
	FILE		*fp;
	NODE_ID		p;
	
	fp = fopen(LOG_FILE_PATH, "w+b");
	if( fp == NULL )
	{
		return;
	}
	
	p = LOG_LIST->List_Head;
	
	while( p!=NULL )
	{
		fwrite(p, sizeof(struct _Node_), 1, fp);
		
		p = p->Next;	
	}
	
	fclose(fp);
}

/*--------------------------------------------------------------
* ROUTINE NAME - Read_File
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void Read_File(void)
{
	FILE			*fp;
	struct _Node_	q;
	
	fp = fopen(LOG_FILE_PATH, "rb");
	if( fp == NULL)
	{
		return;
	}
	
	Free_All_List();
	
	rewind(fp);
	
	while(1)
	{
		fread(&q, sizeof(struct _Node_), 1, fp);
		
		if( feof(fp) != 0 )
		{
			break;
		}
		
#ifdef WDBG
		fprintf(stderr, "%s|%s|%s|%s\n",q.DateTime, q.Message, q.Source, q.Note);
#endif

		Insert_Node(q.DateTime, q.Message, q.Source, q.Note);
	}
	
	fclose(fp);
}

/*--------------------------------------------------------------
* ROUTINE NAME - Log_Time
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void Log_Time(void)
{
	time_t	timep;
	struct tm *p;
	
	time(&timep);
	p=localtime(&timep);
	
	sprintf(DateTime, "%02d/%02d/%d %02d:%02d:%02d", p->tm_mon+1, p->tm_mday, p->tm_year+1900, p->tm_hour, p->tm_min, p->tm_sec);
}

/*--------------------------------------------------------------
* ROUTINE NAME - Add_Login_Msg
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void Add_Login_Msg(char *Client_IP)
{
	Log_Time();
	strcpy(Message, LOGIN_MSG);
	strcpy(Source, Client_IP);
	strcpy(Note, "");
	Insert_Node(DateTime, Message, Source, Note);
	Write_File();
}

/*--------------------------------------------------------------
* ROUTINE NAME - Add_Logout_Msg
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void Add_Logout_Msg(char *Client_IP)
{
	Log_Time();
	strcpy(Message, LOGOUT_MSG);
	strcpy(Source, Client_IP);
	strcpy(Note, "");
	Insert_Node(DateTime, Message, Source, Note);
	Write_File();
}

/*--------------------------------------------------------------
* ROUTINE NAME - Add_PowerOn_Msg
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void Add_PowerOn_Msg(void)
{
	Log_Time();
	strcpy(Message, POWERON_MSG);
	strcpy(Source, "");
	strcpy(Note, "");
	Insert_Node(DateTime, Message, Source, Note);
	Write_File();
}


void Add_Modifypw_Msg(char *Client_IP)
{
	Log_Time();
	strcpy(Message, MODIFYPW_MSG);
	strcpy(Source,Client_IP);
	strcpy(Note,"");
	Insert_Node(DateTime, Message, Source, Note);
	Write_File();
}

