/*
 * main.c -- Main program for the GoAhead WebServer (LINUX version)
 *
 * Copyright (c) GoAhead Software Inc., 1995-2000. All Rights Reserved.
 *
 * See the file "license.txt" for usage and redistribution license requirements
 */

/******************************** Description *********************************/

/*
 *	Main program for for the GoAhead WebServer. This is a demonstration
 *	main program to initialize and configure the web server.
 */

/********************************* Includes ***********************************/
#include	"../uemf.h"
#include	"../wsIntrn.h"
#include	<signal.h>
#include	<unistd.h> 
#include	<sys/types.h>
#include	<sys/wait.h>
#include	"web_api.h"
#include	"vendor.h"

#ifdef WEBS_SSL_SUPPORT
#include	"../websSSL.h"
#endif

#ifdef USER_MANAGEMENT_SUPPORT
#include	"../um.h"
void	formDefineUserMgmt(void);
#endif


/*********************************** Locals ***********************************/
/*
 *	Change configuration here
 */
static char_t		*rootWeb = T("/web");			/* Root web directory */
//static char_t		*password = T("");				/* Security password */
static int			port = 80;						/* Server port */
static int			retries = 5;					/* Server port retries */
static int			finished;						/* Finished flag */


/****************************** Forward Declarations **************************/

static int 	initWebs();
static int  websHomePageHandler(webs_t wp, char_t *urlPrefix, char_t *webDir, int arg, char_t *url, char_t *path, char_t *query);
extern void defaultErrorHandler(int etype, char_t *msg);
extern void defaultTraceHandler(int level, char_t *buf);
#ifdef B_STATS
static void printMemStats(int handle, char_t *fmt, ...);
static void memLeaks();
#endif

extern void formLogin(webs_t wp, char_t *path, char_t *query);
extern void formLogout(webs_t wp, char_t *path, char_t *query);
extern void UserLogout(webs_t wp, char_t *path, char_t *query);

extern void mainMenu(webs_t wp, char_t *path, char_t *query);

extern void cellvisionSetup1(webs_t wp, char_t *path, char_t *query);
//extern void cellvisionSetup2(webs_t wp, char_t *path, char_t *query);
extern void cellvisionSetup3(webs_t wp, char_t *path, char_t *query);
extern void cellvisionSetup4(webs_t wp, char_t *path, char_t *query);
extern void cellvisionSetup5(webs_t wp, char_t *path, char_t *query);

extern void adv_status(webs_t wp, char_t *path, char_t *query);
extern void adv_account(webs_t wp, char_t *path, char_t *query);
extern void adv_samba(webs_t wp, char_t *path, char_t *query);
extern void adv_session(webs_t wp, char_t *path, char_t *query);
extern void adv_security(webs_t wp, char_t *path, char_t *query);

extern void adv_lan(webs_t wp, char_t *path, char_t *query);
extern void adv_timezone(webs_t wp, char_t *path, char_t *query);
extern void adv_adduser(webs_t wp, char_t *path, char_t *query);
extern void adv_remove_user(webs_t wp, char_t *path, char_t *query);
extern void adv_modify_user(webs_t wp, char_t *path, char_t *query);
extern void adv_userlist(webs_t wp, char_t *path, char_t *query);
extern void adv_main_home(webs_t wp, char_t *path, char_t *query);
extern void adv_main_basic(webs_t wp, char_t *path, char_t *query);
extern void adv_main_advanced(webs_t wp, char_t *path, char_t *query);
extern void adv_main_maintenance(webs_t wp, char_t *path, char_t *query);

extern void CfgGeneral(webs_t wp, char_t *path, char_t *query);
extern void CfgIP(webs_t wp, char_t *path, char_t *query);
extern void CfgTime(webs_t wp, char_t *path, char_t *query);
extern void CfgDefault(webs_t wp, char_t *path, char_t *query);
//extern void CfgRestore(webs_t wp, char_t *path, char_t *query);
extern void CfgGlobal(webs_t wp, char_t *path, char_t *query);
extern void CfgSession(webs_t wp, char_t *path, char_t *query);
extern void CfgSecurity(webs_t wp, char_t *path, char_t *query);
extern void CfgFWUpload(webs_t wp, char_t *path, char_t *query);
extern void CfgWriteFlashOK(webs_t wp, char_t *path, char_t *query);

//wireless
extern void adv_wireless(webs_t wp, char_t *path, char_t *query);
extern void wireless_setting(webs_t wp, char_t *path, char_t *query);
extern void Wire_SetSiteSurvey(webs_t wp, char_t *path, char_t *query);
extern void adv_site_survey(webs_t wp, char_t *path, char_t *query);
extern void Wire_ConnectSurvey(webs_t wp, char_t *path, char_t *query);
extern void Wire_AddSurvey(webs_t wp, char_t *path, char_t *query);
extern void WireCountryRegion(webs_t wp, char_t *path, char_t *query);

//card reader backup
extern void adv_cardrd_backup(webs_t wp, char_t *path, char_t *query);
extern void SetCardrd_path(webs_t wp, char_t *path, char_t *query);


extern void mt_idle_time(webs_t wp, char_t *path, char_t *query);
extern void mt_default(webs_t wp, char_t *path, char_t *query);
extern void mtFWUpload(webs_t wp, char_t *path, char_t *query);


//session
extern void adv_add_session(webs_t wp, char_t *path, char_t *query);
extern void adv_modify_session(webs_t wp, char_t *path, char_t *query);
extern void adv_remove_session(webs_t wp, char_t *path, char_t *query);
extern void session_setup(webs_t wp, char_t *path, char_t *query);
extern void del_session(webs_t wp, char_t *path, char_t *query);
extern void get_session_info(webs_t wp, char_t *path, char_t *query);
extern void modify_session(webs_t wp, char_t *path, char_t *query);


// user schedule download
extern void schedule_list(webs_t wp, char_t *path, char_t *query);
extern void user_home(webs_t wp, char_t *path, char_t *query);
extern void adv_d_schedule(webs_t wp, char_t *path, char_t *query);
extern void adv_show_log(webs_t wp, char_t *path, char_t *query);
extern void download_stop(webs_t wp, char_t *path, char_t *query);
extern void download_start(webs_t wp, char_t *path, char_t *query);




//adv_account 
extern void adduser(webs_t wp, char_t * path, char_t * query);
extern void remove_user(webs_t wp, char_t *path, char_t *query);
extern void modify_user(webs_t wp, char_t *path, char_t *query);

extern void download_schedule(webs_t wp, char_t *path, char_t *query);
extern void download_schedule_test(webs_t wp, char_t *path, char_t *query);
extern void del_schedule(webs_t wp, char_t *path, char_t *query);
extern void session_setup(webs_t wp, char_t * path, char_t * query);
extern void right_now_d(webs_t wp, char_t * path, char_t * query);


extern void GetNewDir(webs_t wp, char_t * path, char_t * query);
extern void GetNewDir_All(webs_t wp, char_t * path, char_t * query);

//ftp_server
extern void ftp_setup(webs_t wp, char_t * path, char_t * query);
extern void adv_ftp_status(webs_t wp, char_t * path, char_t * query);
extern void adv_ftp_setting(webs_t wp, char_t * path, char_t * query);
extern void ftp_start_stop(webs_t wp, char_t * path, char_t * query);
extern void ftp_add_path(webs_t wp, char_t * path, char_t * query);
extern void ftp_del_path(webs_t wp, char_t * path, char_t * query);




//Asp function
extern int survey_list(int eid, webs_t wp, int argc, char_t **argv);
extern int getInfo(int eid, webs_t wp, int argc, char_t **argv);
//extern int getInfo2(int eid, webs_t wp, int argc, char_t **argv);
extern int checkInfo(int eid, webs_t wp, int argc, char_t **argv);
extern int getSamba(int eid, webs_t wp, int argc, char_t **argv);
extern int checkInfo2(int eid, webs_t wp, int argc, char_t **argv);

extern int getTime(int eid, webs_t wp, int argc, char_t **argv);
extern int checkTime(int eid, webs_t wp, int argc, char_t **argv);
extern int show_s_log(int eid, webs_t wp, int argc, char_t **argv);

extern int show_modify_user(int eid, webs_t wp, int argc, char_t **argv);
extern int show_remove_user(int eid, webs_t wp, int argc, char_t **argv);
extern int user_list(int eid, webs_t wp, int argc, char_t **argv);
extern int show_s_log(int eid, webs_t wp, int argc, char_t **argv);
extern int show_test_status(int eid, webs_t wp, int argc, char_t **argv);
extern int session_user_list(int eid, webs_t wp, int argc, char_t **argv);
extern int checkTime(int eid, webs_t wp, int argc, char_t **argv);

extern int show_session(int eid, webs_t wp, int argc, char_t **argv);
extern int session_user_list(int eid, webs_t wp, int argc, char_t **argv);
extern int show_modify_session(int eid, webs_t wp, int argc, char_t **argv);
extern int show_session_info(int eid, webs_t wp, int argc, char_t **argv);
extern int show_session_status(int eid, webs_t wp, int argc, char_t **argv);
extern int show_ftp_status(int eid, webs_t wp, int argc, char_t **argv);
extern int show_download_status(int eid, webs_t wp, int argc, char_t **argv);
extern int getVersion(int eid, webs_t wp, int argc, char_t **argv);
extern int Show_AddSession_Status(int eid, webs_t wp, int argc, char_t **argv);


//DLINK
extern void adv_modify_admin(webs_t wp, char_t *path, char_t *query);
extern void DLINK_winzard1(webs_t wp, char_t *path, char_t *query);
extern void DLINK_winzard2(webs_t wp, char_t *path, char_t *query);
extern void DLINK_winzard3(webs_t wp, char_t *path, char_t *query);
extern void DLINK_winzard4(webs_t wp, char_t *path, char_t *query);
extern void DLINK_winzard4_1(webs_t wp, char_t *path, char_t *query);
extern void DLINK_winzard5(webs_t wp, char_t *path, char_t *query);
extern void modify_admin_passwd(webs_t wp, char_t *path, char_t *query);
extern void D_LINKCfgTime(webs_t wp, char_t *path, char_t *query);
extern void adv_wizard(webs_t wp, char_t *path, char_t *query);

extern int show_admin_passwd_message(int eid, webs_t wp, int argc, char_t **argv);
extern int show_add_user_message(int eid, webs_t wp, int argc, char_t **argv);
extern int D_LINK_user_list(int eid, webs_t wp, int argc, char_t **argv);
extern int D_LINK_show_session(int eid, webs_t wp, int argc, char_t **argv);
extern int D_LINK_show_s_log(int eid, webs_t wp, int argc, char_t **argv);
extern int D_LINK_show_s_log2(int eid, webs_t wp, int argc, char_t **argv);
extern int DLINK_show_test_status(int eid, webs_t wp, int argc, char_t **argv);
extern int check_login(int eid, webs_t wp, int argc, char_t **argv);

extern int GetCardrd_path(int eid, webs_t wp, int argc, char_t **argv);
extern int ftp_show_user(int eid, webs_t wp, int argc, char_t **argv);
extern int show_ftp_path(int eid, webs_t wp, int argc, char_t **argv);
extern int ftp_get_status(int eid, webs_t wp, int argc, char_t **argv);

//SiteCom
extern int Go_Index_Page(int eid, webs_t wp, int argc, char_t **argv);

//add user function 
extern int checkID(char *name);
extern int Show_CountUser(int eid, webs_t wp, int argc, char_t **argv);

//wireless
extern int Show_Connetc_Status(int eid, webs_t wp, int argc, char_t **argv);

//schedule downlaod extern function
extern void ClearDTime(void);

//for add anonymous passwd
extern  void SearchFTPSetting(void);


int UpdateSecond = 0; //jack20040409+ for schedule download refresh download log

/*********************************** Code *************************************/
/*
 *	Main -- entry point from LINUX
 */

int main(int argc, char** argv)
{
	FILE	*fp;
/*
 *	Initialize the memory allocator. Allow use of malloc and start 
 *	with a 60K heap.  For each page request approx 8KB is allocated.
 *	60KB allows for several concurrent page requests.  If more space
 *	is required, malloc will be used for the overflow.
 */
	bopen(NULL, (60 * 1024), B_USE_MALLOC);
	signal(SIGPIPE, SIG_IGN);

/*
 *	Initialize the web server
 */
	ClearDTime();
	UpdateSecond = time((time_t *)NULL); //jack20040409+ for schedule download refresh download log
//	SearchFTPSetting();  //jack20040330
	
	
	if (initWebs() < 0) {
		return -1;
	}
	
#ifdef DLINK
	system("touch /default/ftp_tbl");
	system("mount -t minix /dev/mtdblock1 /sys/mtd1");
	system("cp -f /sys/mtd1/ftp_tbl /etc/");
	system("sync");
	system("umount /sys/mtd1");
#endif
		
#ifdef WEBS_SSL_SUPPORT
	websSSLOpen();
#endif


/*
 *	Basic event loop. SocketReady returns true when a socket is ready for
 *	service. SocketSelect will block until an event occurs. SocketProcess
 *	will actually do the servicing.
 */
	while (!finished) {
		if (socketReady(-1) || socketSelect(-1, 1000)) {
			socketProcess(-1);
		}
		websCgiCleanup();
		emfSchedProcess();
	}

#ifdef WEBS_SSL_SUPPORT
	websSSLClose();
#endif

#ifdef USER_MANAGEMENT_SUPPORT
	umClose();
#endif

/*
 *	Close the socket module, report memory leaks and close the memory allocator
 */
	websCloseServer();
	socketClose();
#ifdef B_STATS
	memLeaks();
#endif
	bclose();
	return 0;
}
/******************************************************************************/
/*
 *	Initialize the web server.
 */

static int initWebs()
{
	struct hostent	*hp;
	struct in_addr	intaddr;
	char			host[128];// dir[128], webdir[128];
	char			*cp;
	char_t			wbuf[128];
	FILE *fp,*fp2;
	char s[80],line[80];
	char buf[80],buf2[32];
	char	*p1,*p2,*p3,*p4;
	char	autoIP[16];
	int	flag=1,tmp=0;
	int len;
	

/*
 *	Initialize the socket subsystem
 */
	socketOpen();

#ifdef USER_MANAGEMENT_SUPPORT
/*
 *	Initialize the User Management database
 */
	umOpen();
	umRestore(T("umconfig.txt"));
#endif


/*
 *	Define the local Ip address, host name, default home page and the 
 *	root web directory.
 */
 	//fish2040729+ set Daylight
	fp2 = fopen("/web/sib.conf","r");
	while((fgets(line,80,fp2)) != NULL)	
	{
		if (strstr(line,"CF-Daylight"))
		{
			p3=strchr(line,'"');
			p4=rindex(line,'"');
			*p4=0x00;
			sprintf(buf2,"/usr/sbin/daylight %s &",p3+1);
			printf ("Set Daylight :%s\n",buf2);
			system(buf2);
		}
	}
	fclose(fp2);
	
	if (gethostname(host, sizeof(host)) < 0) {
		error(E_L, E_LOG, T("Can't get hostname"));
		return -1;
	}

	if ((hp = gethostbyname(host)) == NULL) 
	{
		
		while(flag==1)
		{
			system("ifconfig > /web/autoIP");
			fp = fopen("/web/autoIP","r");
			while((fgets(s,80,fp)) != NULL)	
			{		
				if (strstr(s,"br0:0"))
				{
					tmp=1;
					continue;
				}
				
				if (tmp==1)
				{
					if (p1=strstr(s,"inet addr:"))
					{
						p1=p1+10;
						p2=strstr(p1," ");
						*p2 = 0x00;
						strcpy(autoIP , p1);
						flag=0;
						//printf ("autoIP=%s\n",autoIP);
						break;					
					}
				}
				else
				{
					flag=1;
				}

			}
			sleep(1);
			fclose(fp);
		}
		if (flag==1)
		{
		error(E_L, E_LOG, T("Can't get host address"));
			printf("Can't get host address\n");
		return -1;
	}
		
		
	}
	if (flag==0)
	{
		memcpy((char *) &intaddr,autoIP,strlen(autoIP));
		fp = fopen("/etc/hosts","r");
		fgets(s,80,fp);
		fgets(s,80,fp);
		fclose(fp);
		p1 = strrchr(s,' ');
		p1++;
		len = strlen(p1);
		if ( p1[len-1] == 0x0a )
			p1[len-1] =0x00;
		
		fp = fopen("/etc/hosts","w+");
		fputs("127.0.0.1 localhost.localdomain localhost\n",fp);
		sprintf(buf,"%s %s %s\n\n",autoIP,p1,p1);
		fputs(buf,fp);
		fclose(fp);
		system("smb restart");
		
	}
	else
	{	
	memcpy((char *) &intaddr, (char *) hp->h_addr_list[0], (size_t) hp->h_length);
	}



/*
 *	Set ../web as the root web. Modify this to suit your needs
 */
//royc modify
//	getcwd(dir, sizeof(dir)); 
//	if ((cp = strrchr(dir, '/'))) {
//		*cp = '\0';
//	}
//	sprintf(webdir, "%s/%s", dir, rootWeb);
//royc end
	
//fprintf(stderr, "rootWeb:%s\n", rootWeb);

/*
 *	Configure the web server options before opening the web server
 */
	//websSetDefaultDir(webdir);	//royc modify root dir
	websSetDefaultDir(rootWeb);		//royc add
	cp = inet_ntoa(intaddr);
	ascToUni(wbuf, cp, min(strlen(cp) + 1, sizeof(wbuf)));
	websSetIpaddr(wbuf);
	ascToUni(wbuf, host, min(strlen(host) + 1, sizeof(wbuf)));
	websSetHost(wbuf);

/*
 *	Configure the web server options before opening the web server
 */
	websSetDefaultPage(T("login.asp"));
	//websSetPassword(password);

/* 
 *	Open the web server on the given port. If that port is taken, try
 *	the next sequential port for up to "retries" attempts.
 */
	websOpenServer(port, retries);

/*
 * 	First create the URL handlers. Note: handlers are called in sorted order
 *	with the longest path handler examined first. Here we define the security 
 *	handler, forms handler and the default web page handler.
 */
	websUrlHandlerDefine(T(""), NULL, 0, websSecurityHandler, WEBS_HANDLER_FIRST);
	websUrlHandlerDefine(T("/goform"), NULL, 0, websFormHandler, 0);
	websUrlHandlerDefine(T("/cgi-bin"), NULL, 0, websCgiHandler, 0);
	websUrlHandlerDefine(T(""), NULL, 0, websDefaultHandler, WEBS_HANDLER_LAST); 

/*
 *	Now define two test procedures. Replace these with your application
 *	relevant ASP script procedures and form functions.
 */
	websFormDefine(T("formLogin"), formLogin);
	websFormDefine(T("mainMenu"), mainMenu);
	websFormDefine(T("formLogout"), formLogout);
	websFormDefine(T("UserLogout"), UserLogout);
	websFormDefine(T("CfgTime"), CfgTime);
	websFormDefine(T("CfgIP"), CfgIP);
	websFormDefine(T("CfgGeneral"), CfgGeneral);
	websFormDefine(T("cellvisionSetup1"), cellvisionSetup1);
	websFormDefine(T("cellvisionSetup3"), cellvisionSetup3);
	websFormDefine(T("cellvisionSetup4"), cellvisionSetup4);
	websFormDefine(T("cellvisionSetup5"), cellvisionSetup5);
	websFormDefine(T("CfgDefault"), CfgDefault);
	websFormDefine(T("CfgGlobal"), CfgGlobal);
	websFormDefine(T("CfgSession"), CfgSession);
	websFormDefine(T("CfgSecurity"), CfgSecurity);
	websFormDefine(T("adv_status"), adv_status);
	websFormDefine(T("adv_account"), adv_account);
	websFormDefine(T("adv_samba"), adv_samba);
	websFormDefine(T("adv_session"), adv_session);
	websFormDefine(T("adv_security"), adv_security);
	websFormDefine(T("adv_lan"), adv_lan);
	websFormDefine(T("adv_timezone"), adv_timezone);
	websFormDefine(T("mt_idle_time"), mt_idle_time);
	websFormDefine(T("mt_default"), mt_default);
	websFormDefine(T("mtFWUpload"), mtFWUpload);
	websFormDefine(T("CfgFWUpload"), CfgFWUpload);
	websFormDefine(T("CfgWriteFlashOK"), CfgWriteFlashOK);
	websFormDefine(T("schedule_list"), schedule_list);
	websFormDefine(T("user_home"), user_home);
	websFormDefine(T("adv_d_schedule"), adv_d_schedule);
	websFormDefine(T("adv_show_log"), adv_show_log);
	websFormDefine(T("adv_add_session"), adv_add_session);
	websFormDefine(T("adv_modify_session"), adv_modify_session);
	websFormDefine(T("adv_remove_session"), adv_remove_session);
	websFormDefine(T("right_now_d"), right_now_d);
	websFormDefine(T("ftp_setup"), ftp_setup);
	websFormDefine(T("adv_ftp_status"), adv_ftp_status);
	websFormDefine(T("adv_ftp_setting"), adv_ftp_setting);
	websFormDefine(T("download_stop"), download_stop);
	websFormDefine(T("download_start"), download_start);
	websFormDefine(T("adv_main_home"), adv_main_home);
	websFormDefine(T("adv_main_basic"), adv_main_basic);
	websFormDefine(T("adv_main_advanced"), adv_main_advanced);
	websFormDefine(T("adv_main_maintenance"), adv_main_maintenance);
	
	//wireless
	websFormDefine(T("adv_wireless"), adv_wireless);
	websFormDefine(T("wireless_setting"), wireless_setting);
	websFormDefine(T("adv_site_survey"), adv_site_survey);
	websFormDefine(T("Wire_ConnectSurvey"), Wire_ConnectSurvey);	
	websFormDefine(T("Wire_SetSiteSurvey"), Wire_SetSiteSurvey);	
	websFormDefine(T("Wire_AddSurvey"), Wire_AddSurvey);
	websFormDefine(T("WireCountryRegion"), WireCountryRegion);

	//card reader backup
	websFormDefine(T("adv_cardrd_backup"), adv_cardrd_backup);
	websFormDefine(T("SetCardrd_path"), SetCardrd_path);


	// adduser
	websFormDefine(T("adduser"), adduser);
	websFormDefine(T("adv_adduser"), adv_adduser);
	websFormDefine(T("adv_remove_user"), adv_remove_user);
	websFormDefine(T("remove_user"), remove_user);
	websFormDefine(T("adv_modify_user"), adv_modify_user);	
	websFormDefine(T("modify_user"), modify_user);
	websFormDefine(T("adv_userlist"), adv_userlist);	
	websFormDefine(T("download_schedule"), download_schedule);
	websFormDefine(T("download_schedule_test"), download_schedule_test);
	websFormDefine(T("del_schedule"), del_schedule);
	websFormDefine(T("session_setup"), session_setup);
	websFormDefine(T("del_session"), del_session);
	websFormDefine(T("get_session_info"), get_session_info);
	websFormDefine(T("modify_session"), modify_session);
	
	websFormDefine(T("GetNewDir"), GetNewDir);
	websFormDefine(T("GetNewDir_All"), GetNewDir_All);
	websFormDefine(T("ftp_start_stop"), ftp_start_stop);
	websFormDefine(T("ftp_add_path"), ftp_add_path);
	websFormDefine(T("ftp_del_path"), ftp_del_path);

	
	websAspDefine(T("survey_list"), survey_list);
	websAspDefine(T("getSamba"), getSamba);
	websAspDefine(T("getInfo"), getInfo);
	websAspDefine(T("checkInfo"), checkInfo);
	websAspDefine(T("checkInfo2"), checkInfo2);
	websAspDefine(T("getTime"), getTime);
	websAspDefine(T("checkTime"), checkTime);
	websAspDefine(T("show_s_log"), show_s_log);
	websAspDefine(T("show_modify_user"), show_modify_user);
    	websAspDefine(T("user_list"), user_list);
	websAspDefine(T("show_remove_user"), show_remove_user);
	websAspDefine(T("show_test_status"), show_test_status);
	websAspDefine(T("session_user_list"), session_user_list);
	websAspDefine(T("show_session"), show_session);
	websAspDefine(T("show_modify_session"), show_modify_session);
	websAspDefine(T("show_session_info"), show_session_info);
	websAspDefine(T("show_session_status"), show_session_status);
	websAspDefine(T("show_ftp_status"), show_ftp_status);
	websAspDefine(T("show_download_status"), show_download_status);
	websAspDefine(T("getVersion"), getVersion);
	websAspDefine(T("Show_AddSession_Status"), Show_AddSession_Status);
	websAspDefine(T("check_login"), check_login);
	
	
//DLINK
	websFormDefine(T("adv_modify_admin"), adv_modify_admin);
	websFormDefine(T("DLINK_winzard1"), DLINK_winzard1);
	websFormDefine(T("DLINK_winzard2"), DLINK_winzard2);
	websFormDefine(T("DLINK_winzard3"), DLINK_winzard3);
	websFormDefine(T("DLINK_winzard4"), DLINK_winzard4);
	websFormDefine(T("DLINK_winzard4_1"), DLINK_winzard4_1);
	websFormDefine(T("DLINK_winzard5"), DLINK_winzard5);
	websFormDefine(T("modify_admin_passwd"), modify_admin_passwd);
	websFormDefine(T("D_LINKCfgTime"), D_LINKCfgTime);
	websFormDefine(T("adv_wizard"), adv_wizard);


	websAspDefine(T("show_admin_passwd_message"), show_admin_passwd_message);
	websAspDefine(T("show_add_user_message"), show_add_user_message);
	websAspDefine(T("D_LINK_user_list"), D_LINK_user_list);
	websAspDefine(T("D_LINK_show_session"), D_LINK_show_session);
	websAspDefine(T("D_LINK_show_s_log"), D_LINK_show_s_log);
	websAspDefine(T("D_LINK_show_s_log2"), D_LINK_show_s_log2);
	websAspDefine(T("DLINK_show_test_status"), DLINK_show_test_status);


	websAspDefine(T("GetCardrd_path"), GetCardrd_path);
	websAspDefine(T("ftp_show_user"), ftp_show_user);
	websAspDefine(T("show_ftp_path"), show_ftp_path);
	websAspDefine(T("ftp_get_status"), ftp_get_status);
	
	
//SiteCom
	websAspDefine(T("Go_Index_Page"), Go_Index_Page);
	
//Lobos
	websAspDefine(T("Show_CountUser"), Show_CountUser);

//wireless
	websAspDefine(T("Show_Connetc_Status"), Show_Connetc_Status);
/*
 *	Create the Form handlers for the User Management pages
 */
#ifdef USER_MANAGEMENT_SUPPORT
	formDefineUserMgmt();
#endif

/*
 *	Create a handler for the default home page
 */
	websUrlHandlerDefine(T("/"), NULL, 0, websHomePageHandler, 0); 
	return 0;
}

/******************************************************************************/
/*
 *	Test Javascript binding for ASP. This will be invoked when "aspTest" is
 *	embedded in an ASP page. See web/asp.asp for usage. Set browser to 
 *	"localhost/asp.asp" to test.
 */

//static int aspTest(int eid, webs_t wp, int argc, char_t **argv)
//{
//	char_t	*name, *address;
//
//	if (ejArgs(argc, argv, T("%s %s"), &name, &address) < 2) {
//		websError(wp, 400, T("Insufficient args\n"));
//		return -1;
//	}
//	return websWrite(wp, T("Name: %s, Address %s"), name, address);
//}

/******************************************************************************/
/*
 *	Test form for posted data (in-memory CGI). This will be called when the
 *	form in web/asp.asp is invoked. Set browser to "localhost/asp.asp" to test.
 */

//static void formTest(webs_t wp, char_t *path, char_t *query)
//{
//	char_t	*name, *address;
//
//	name = websGetVar(wp, T("name"), T("Joe Smith")); 
//	address = websGetVar(wp, T("address"), T("1212 Milky Way Ave.")); 
//
//	websHeader(wp);
//	websWrite(wp, T("<body><h2>Name: %s, Address: %s</h2>\n"), name, address);
//	websFooter(wp);
//	websDone(wp, 200);
//}

/******************************************************************************/
/*
 *	Home page handler
 */

static int websHomePageHandler(webs_t wp, char_t *urlPrefix, char_t *webDir,
	int arg, char_t *url, char_t *path, char_t *query)
{
/*
 *	If the empty or "/" URL is invoked, redirect default URLs to the home page
 */
	if (*url == '\0' || gstrcmp(url, T("/")) == 0) {
        //websRedirect(wp, T("upload.htm"));
        websRedirect(wp, T("/web/login.asp"));
		return 1;
	}
	return 0;
}

/******************************************************************************/
/*
 *	Default error handler.  The developer should insert code to handle
 *	error messages in the desired manner.
 */

void defaultErrorHandler(int etype, char_t *msg)
{
#if 0
	write(1, msg, gstrlen(msg));
#endif
}

/******************************************************************************/
/*
 *	Trace log. Customize this function to log trace output
 */

void defaultTraceHandler(int level, char_t *buf)
{
/*
 *	The following code would write all trace regardless of level
 *	to stdout.
 */
#if 0
	if (buf) {
		write(1, buf, gstrlen(buf));
	}
#endif
}

/******************************************************************************/
/*
 *	Returns a pointer to an allocated qualified unique temporary file name.
 *	This filename must eventually be deleted with bfree();
 */

char_t *websGetCgiCommName()
{
	char_t	*pname1, *pname2;

	//pname1 = tempnam(NULL, T("cgi"));
	pname1 = (char*)mkstemp(T("cgi"));
	pname2 = bstrdup(B_L, pname1);
	free(pname1);
	return pname2;
}

/******************************************************************************/
/*
 *	Launch the CGI process and return a handle to it.
 */

int websLaunchCgiProc(char_t *cgiPath, char_t **argp, char_t **envp,
					  char_t *stdIn, char_t *stdOut)
{
	int	pid, fdin, fdout, hstdin, hstdout, rc;

	fdin = fdout = hstdin = hstdout = rc = -1; 
	if ((fdin = open(stdIn, O_RDWR | O_CREAT, 0666)) < 0 ||
		(fdout = open(stdOut, O_RDWR | O_CREAT, 0666)) < 0 ||
		(hstdin = dup(0)) == -1 ||
		(hstdout = dup(1)) == -1 ||
		dup2(fdin, 0) == -1 ||
		dup2(fdout, 1) == -1) {
		goto DONE;
	}
		
 	rc = pid = fork();
 	if (pid == 0) {
/*
 *		if pid == 0, then we are in the child process
 */
		if (execve(cgiPath, argp, envp) == -1) {
			printf("content-type: text/html\n\n"
				"Execution of cgi process failed\n");
		}
		exit (0);
	} 

DONE:
	if (hstdout >= 0) {
		dup2(hstdout, 1);
	}
	if (hstdin >= 0) {
		dup2(hstdin, 0);
	}
	if (fdout >= 0) {
		close(fdout);
	}
	if (fdin >= 0) {
		close(fdin);
	}
	return rc;
}

/******************************************************************************/
/*
 *	Check the CGI process.  Return 0 if it does not exist; non 0 if it does.
 */

int websCheckCgiProc(int handle)
{
/*
 *	Check to see if the CGI child process has terminated or not yet.  
 */
	if (waitpid(handle, NULL, WNOHANG) == handle) {
		return 0;
	} else {
		return 1;
	}
}

/******************************************************************************/

#ifdef B_STATS
static void memLeaks() 
{
	int		fd;

	if ((fd = gopen(T("leak.txt"), O_CREAT | O_TRUNC | O_WRONLY)) >= 0) {
		bstats(fd, printMemStats);
		close(fd);
	}
}

/******************************************************************************/
/*
 *	Print memory usage / leaks
 */

static void printMemStats(int handle, char_t *fmt, ...)
{
	va_list		args;
	char_t		buf[256];

	va_start(args, fmt);
	vsprintf(buf, fmt, args);
	va_end(args);
	write(handle, buf, strlen(buf));
}
#endif

/******************************************************************************/

