
/*****************************************************************************/ 
/*  File Name: s_loglist.c                                                   */
/*  File Description: recode download schedule file                          */
/*                                                                           */
/*  History:                                                                 */
/*      Date     Author   Version  Description                               */
/*   ----------------------------------------------------------------------  */
/*				jack	1.00	1. First release							 */
/*  2004/03/15	jack	1.01    1.add new InitDownloadAT function			 */
/*								(jack20040315)								 */
/*	                            2.add new CheckDownloadSchedule function	 */
/*								(jack20040315)							 	 */
/*								3.add Get_at_work_ID function form        	 */
/*                                d_schedule.c(jack20040315)				 */
/*                              4.add CheckDownloadSafe function form		 */
/*								 d_schedule.c (jack20040315)    			 */
/*  2004/03/17   jack	1.02    1.add schedule download limit           	 */
/*                               (search "jack20040315" )					 */
/*								2.add CheckDownloadSafe function to check	 */
/*                      		 file name if same(jack20040317)			 */
/*								3.add SubName member in NODE_ID struct		 */
/*								  for download file name same need to change */
/*								  file name (jack20040318)					 */
/*                              4.prevent to del wget log when file is    	 */
/*								  downloading (jack20040319)				 */
/*  2004/03/30	jack	1.03	1.for downlaod file subname , if download 	 */
/*								  file is now downloading then add subname	 */
/*								  (jack20040330)							 */
/*	2004/04/06	jack	1.04	1.for schedule download file wait		 	 */
/*								  (jack20040406)							 */
/*								2.for downloading file unplug usb device 	 */
/*								  (jack20040406)							 */
/*	2004/04/07	jack			1.for wait wget.log to record (jack20040407) */
/*  2004/04/08	jack			1.change method to write log (jack20040408)	 */
/*								2.lock wget.log when del user log 			 */
/*								(jack20040408)								 */
/*								3.update re-schedule download time when 	 */
/*								 re-download(jack20040408)					 */ 
/*								4.add del download when status is now		 */
/*								  download(jack20040408)					 */
/*	2004/10/07	jack			1.modify for backup to remote PC			 */
/*								(jack20041007)								 */
/*	2004/11/17	jack			1.for dobule check download if success		 */
/*								(jack20041117)								 */
/*****************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <sys/stat.h>
#include <unistd.h>
#include "s_loglist.h"
#include "debug.h"

extern unsigned short Get_at_work_ID(void);  

//jack20040315+
extern void SetSchedule( char *S_FileName ,
                      char *Data , 
                      char *Time  );                   
              
extern void InitDownloadAT( char *S_FileName , char *buf1 , char *buf , char *Directory , char DownloadDIR );
extern void InitBackupAT( char *S_BACKUP , char *cmd , char *data , char *Destination , char *URL , char BackupToPC );
extern void GetDateTime(char *DateTime );
extern short GetReScheduleCommand( char *Info , char *buf1 , char *DownloadDIR );

void CheckDownloadSchedule(void);

//jack20040315 end

int CheckDownloadSafe( char *O_Path , char *FileName , char *NewSubName , int Type ); //jack20040317+
//int GetSetInfo( char *S_FileName , char *Output , int Type );

void Remove_Node(void);
static NODE_ID Get_Node(void);
static int Is_Empty(void);
void ClearUserNodeStruct(char *Info);
void Read_block( char *buf ,unsigned long offset , int length );
void Write_block( char *buf ,unsigned long offset , int length );
void Record_Status( char *Info );
short ReadUserNode(char *Info , char *S_FileName );
void WriteUserNode(char *Info , char *S_FileName );
void PrintUserNode(char *Info);
short Inc_Download_Num(void);
short Dec_Download_Num(void);

#ifdef LOG_DEBUG
void Debug_Print(char *buf);
#endif

//global var for log list
int Total_Download_File_Num;
LOG_LIST_ID		LOG_LIST = NULL;
char GS_FileName[16]="";


/*--------------------------------------------------------------
* ROUTINE NAME - Init_Log_List
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void Init_Log_List(void)
{
	if( LOG_LIST == NULL )
	{
		//alloc memory
		if( !(LOG_LIST = (LOG_LIST_ID)malloc(sizeof(struct _LOG_LIST_))) )
		{
			return;
		}

		LOG_LIST->Log_Number = 0;
		LOG_LIST->List_Head  = NULL;
		LOG_LIST->List_Tail  = NULL;
	}
}

/*--------------------------------------------------------------
* ROUTINE NAME - Get_Node
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
static NODE_ID Get_Node(void)
{
	NODE_ID		p;

	p = (NODE_ID)malloc(sizeof(struct _Node_));

	if( p == NULL )
	{
		return NULL;
	}
	else
	{
		memset(p , 0 , sizeof( struct _Node_ ));
		return p;
	}
}

/*--------------------------------------------------------------
* ROUTINE NAME - Is_Empty
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
static int Is_Empty(void)
{
	return (LOG_LIST->Log_Number == 0) ? 1 : 0;
}

/*--------------------------------------------------------------
* ROUTINE NAME - Insert_Node
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT: Info --> User Node struct 
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void Insert_Node(char *Info)
{
	NODE_ID		p;
	NODE_ID pNode = ( struct _Node_ * )Info;
	
	p = Get_Node();
	if( p == NULL )
	{
		return;
	}

	if( Is_Empty() )
	{
		//list is empty, just add a node is ok

		LOG_LIST->Log_Number ++;
		LOG_LIST->List_Head = LOG_LIST->List_Tail = p;
	}
	else
	{
		//check max log list number
		if( LOG_LIST->Log_Number >= MAX_LOG_LIST_NUM )
		{
			Remove_Node();
		}

		//list not empty, we add at tail
		LOG_LIST->Log_Number ++;
		LOG_LIST->List_Tail->Next = p;
		LOG_LIST->List_Tail = p;
	}

	//copy value to node
	memcpy( p , pNode , sizeof(struct _Node_)-4 );
	p->Next = NULL;
}

/*--------------------------------------------------------------
* ROUTINE NAME - Remove_Node
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void Remove_Node(void)
{
	NODE_ID		p;

	//we always remove node at head of list
	if( Is_Empty() )
	{
		return;
	}
	else
	{
		p = LOG_LIST->List_Head;

		if( LOG_LIST->Log_Number == 1 )
		{
			LOG_LIST->Log_Number = 0;
			LOG_LIST->List_Head  = NULL;
			LOG_LIST->List_Tail  = NULL;
			free(p);
			return;
		}
		else
		{
			LOG_LIST->Log_Number --;
			LOG_LIST->List_Head = p->Next;
			free(p);
			return;
		}
	}
}

/*--------------------------------------------------------------
* ROUTINE NAME - Free_All_List
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void Free_All_List(void)
{
	while( !Is_Empty() )
	{
		Remove_Node();
	}
}

/*--------------------------------------------------------------
* ROUTINE NAME - Wriet_File
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void Write_File(int del_flag)
{
	FILE		*fp;
	NODE_ID		p;
	char buf[32]="";
//	char max_dl_num[8]="";
	char S_FileName[16]="";
	unsigned short wget_pid = 0;
	char kill_process = 0;
	char kill_SCHEDULE_flag = 0;
	
	fp = fopen(S_LOG_FILE_PATH, "w+b");
	if( fp == NULL )
	{
		return;
	}
	
	p = LOG_LIST->List_Head;

	while( p!=NULL )
	{
		if ( del_flag == 1)
		{
			if ( strcmp( GS_FileName , p->S_FileName ) )
			{
				fwrite(p, sizeof(struct _Node_), 1, fp);
			}
			else
			{
				//jack20040319+ for prevent to del wget log when file is downloading
				//if ( p->Status == 3 ) //downloading
				//jack20040408+ add del download when status is downloading
				if( (p->PID != 0 ) && ( p->Status == 3 ) )
				{
					kill_process = 1;
					wget_pid = p->PID;
				}
				
				if( (p->PID == 0 ) && ( p->Status == 3 ) )
				{
					fwrite(p, sizeof(struct _Node_), 1, fp);
				}
				//jack20040408 end
				//jack20040319 end
				
				//jack20040603+ for del SCHEDULE file
				if ( p->Status == 2 )
				{
					kill_SCHEDULE_flag =1;
					strcpy( S_FileName , p->S_FileName);
				} 
			}
		}
		else
		{
			fwrite(p, sizeof(struct _Node_), 1, fp);
		}
		p = p->Next;
	}
	
	fclose(fp);
	
	//jack20040603+ for del SCHEDULE file
	if ( kill_SCHEDULE_flag == 1 )
	{
		sprintf(buf , "rm /web/%s &",S_FileName);
		system(buf);
	}
	
	if ( kill_process == 1 )
	{
		sprintf(buf , "kill %d &",wget_pid);
		system(buf);
		Dec_Download_Num();
		CheckDownloadSchedule();
	}
	else
	{
//		fp1 = fopen("/tmp/max_dl_num","r+");
//	
//		fgets( max_dl_num , 8 , fp1 );
//		if ( max_dl_num[0] < 0x33 )
//		{
//			max_dl_num[0]++;
//			fseek(fp1 , -1 , 1 );
//			fputs( max_dl_num , fp1);
//			fclose(fp1);
//		}
	}
	
}

/*--------------------------------------------------------------
* ROUTINE NAME - Read_block
*---------------------------------------------------------------
* FUNCTION:2004.05.08 add
*
* INPUT:	buf 	:read date
*			offset	:file point offset
*			length	:write date length
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void Read_block( char *buf ,unsigned long offset , int length )
{
	FILE *fp;
	
	if (( fp = fopen( S_LOG_FILE_PATH ,"rb")) != NULL )
	{
		fseek(fp , offset , 1 );
		fread( (void *)buf , length , 1 , fp );
		fclose(fp);
	}
}

/*--------------------------------------------------------------
* ROUTINE NAME - Write_block
*---------------------------------------------------------------
* FUNCTION:2004.04.08 add
*
* INPUT:	buf 	:write date
*			offset	:file point offset
*			length	:write date length
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void Write_block( char *buf ,unsigned long offset , int length )
{
	FILE *fp;
	
	fp = fopen( S_LOG_FILE_PATH ,"r+b");
	fseek(fp , offset , 1 );
	fwrite( (void *)buf , length , 1 , fp );
	fclose(fp);
	
}

/*--------------------------------------------------------------
* ROUTINE NAME - Write_Add_log
*---------------------------------------------------------------
* FUNCTION:2004.04.08 add
*
* INPUT:	buf 	:write date
*			offset	:file point offset
*			length	:write date length
*			log_count : log count
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void Write_Add_log( char *buf , unsigned long offset , int length , int log_count )
{
	FILE *fp;
	
	if ( log_count == 0 )
	{
		fp = fopen( S_LOG_FILE_PATH , "w+b");
	}
	else
	{
		fp = fopen( S_LOG_FILE_PATH ,"a+b");
		fseek(fp , offset , 1 );
	}
	
	fwrite( (void *)buf , length , 1 , fp );
	fclose(fp);
}

/*--------------------------------------------------------------
* ROUTINE NAME - Read_File
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:
*
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
int Read_File(void)
{
	long log_count = 0;
	FILE			*fp;
	struct _Node_	q;
	NODE_ID pNode = (NODE_ID)(char *)(&q);

	fp = fopen(S_LOG_FILE_PATH, "rb");
	if( fp == NULL)
	{
		return 0;
	}
	
	Free_All_List();
	
	rewind(fp);
	
	while(1)
	{
		fread(&q, sizeof(struct _Node_), 1, fp);
		
		if( feof(fp) != 0 )
		{
			break;
		}
		log_count++;
		//jack20040315+ mofiyf add wait_status paramenter
		Insert_Node((char*)pNode);
	}
	
	fclose(fp);
	
	return log_count;
}

/*--------------------------------------------------------------
* ROUTINE NAME - Record_Status
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:	LoginUser1
*			work_id      --> at work id
*			Status       --> download status
*           wait_status  0 : no need to re-set schedule download
*                        1 : need to re-set schedule download
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void Record_Status( char *Info )
{

//jack20040408+ change method to write log	
	unsigned long offset = 0;
	int log_count = 0;
	
//	printf("size = %d\n",sizeof(struct _Node_));  //total size 404
	
	Init_Log_List();
    log_count = Read_File();
    Remove_Node();
    
//jack20040408+ change method to write log    
    offset = sizeof(struct _Node_)*log_count;
    Write_Add_log( (char *)Info , offset , sizeof(struct _Node_) , log_count );
//jack20040408 end

}


/*--------------------------------------------------------------
* ROUTINE NAME - Record_SCHEDULE_Status
*---------------------------------------------------------------
* FUNCTION:
*
* INPUT:	S_FileName
*			Status       --> download status
*           Type     0: --> no check schedule download 
* OUTPUT:   None     1: --> check
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/

void Record_SCHEDULE_Status( char *S_FileName , long Length , unsigned short Status , int Type  , unsigned short wget_pid )
{
	FILE *fp;
	struct _Node_	User_Node;
    NODE_ID		pNode;
//	NODE_ID		p;
//	short count=0;
//	char max_dl_num[8]="";
	char DownloadName[256];
	char Path[256];
	struct stat buf;
	char *sp;
	
	char SearchFlag = 0;
//jack20041117 for dobule check download if success	
	char ErrorFlag = 0;
	short	len =0;
//jack20041117 end
	short	i,count;	//jack20041123
	
	#ifdef SCHEDULE_DEBUG
	printf("entry Record_SCHEDULE_Status\n");
	#endif
	
//	printf("S_FileName = %s\n",S_FileName);
//	printf("Status = %d\n",Status);
//	printf("Type = %d\n",Type);
//	printf("wget_pid = %d\n",wget_pid);
	
	if ( ( fp = fopen(S_LOG_FILE_PATH,"r") ) == NULL )
	{
		return;
	}
	fclose(fp); //jack20041117
	
	pNode = (NODE_ID)(char *)(&User_Node);
	
	
	if (!ReadUserNode((char*)pNode , S_FileName ) )
	{
    	SearchFlag = 1;
    	//jack20040317+ check file if download finish
    	if( ( Status == 0 ) && ( pNode->CheckDownloadFlag == 0 ) )
    	{
    		pNode->CheckDownloadFlag = 1;
    		
    		if ( pNode->Length > 0 )
    		{
	    		sp = strrchr(pNode->URL ,'/');
	    		sp++;
	    		strcpy( DownloadName , sp );
	    		
	    		sprintf( Path ,"%s%s%s", pNode->O_Directory , DownloadName , pNode->SubName );
	    		
	    		if( stat(Path , &buf ) == 0 )
	    		{
	    			if( ( buf.st_size == 0 ) || ( buf.st_size != pNode->Length ) )
	    			{
	    				ErrorFlag = -1; //FAIL //jack20041117
	    			}
	    		}
	    		else
	    		{
	    			//jack20040406+ for downloading file unplug usb device
	    			ErrorFlag = -1; //FAIL //jack20041117
	    		}	
    		}
    		else 
    		{
    			
    			//jack20041117 for dobule check download if success
    			len = strlen(pNode->URL);
    			count =0;
    			for( i =0 ; i<len ; i++)
    			{
    				if ( pNode->URL[i] == '/' )
    					count++; //count '/' number
    			}
    			
    			if( ( pNode->URL[len-1] == '/' ) && ( count > 3 ) )
    			{
    				
    				pNode->URL[len-1] = 0x00;
    				if( (sp = strrchr(pNode->URL , '/')) != NULL )
    				{
    					sp++;
    					strcpy(Path , pNode->O_Directory);
    					strcat(Path , sp );
    					#ifdef SCHEDULE_DEBUG
    					printf("double check download folder path = %s\n",Path );
    					#endif
    					if ( ( stat( Path , &buf ) ) != 0 )
					 	{
					 		ErrorFlag = -1;
						}
						else
						{
							#ifdef SCHEDULE_DEBUG
							printf("Double check download Folder ok.\n");
							#endif
						}
	    			}//end if( (sp = strrchr(pNode->URL , '/')) != NULL )
	    			pNode->URL[len-1] = '/';	//jack20041123
    			}//end if( ( pNode->URL[len-1] == '/' ) && ( count > 3 ) )
    			//jack20041117 end
    		}//end if ( pNode->Length > 0 )
    	}//end if( ( Status == 0 ) && ( pNode->CheckDownloadFlag == 0 ) )
	}//end if (!ReadUserNode((char*)pNode , S_FileName ) )
	//jack20040317 end

//    	//jack20040406+ for schedule download file wait
//    	if ( pNode->Status == 4 )
//    	{
//    		pNode->wait_status = 1;
//    	}
//    	//jack20040406 end

	
	
	
//jack20040408+ change method to write log	
	//upadte status
	if ( SearchFlag == 1 )
	{
		if( ( wget_pid != 0  ) && ( Type ==2 ) )
		{	
			pNode->PID = wget_pid;
		}
		
		if ( Type == 0 )
		{
		pNode->Length = Length;
		}
		
		//jack20041117 for dobule check download if success
		if ( ErrorFlag == 0 )
			pNode->Status = Status;
		else
			pNode->Status = -1;
		
		WriteUserNode( (char*)pNode, S_FileName );
//jack20040408 end
		
	//jack20040315+	check current wget number
		if ( Type == 1 )
		{
			Dec_Download_Num();
			CheckDownloadSchedule();	
		}
	}

//jack20040315
	
}

/*--------------------------------------------------------------
* ROUTINE NAME - CheckDownloadSchedule
*---------------------------------------------------------------
* FUNCTION:  2004/03/15 add
*			check wget.log , if need to do
* INPUT:	S_FileName
*			
* OUTPUT:   None   
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void CheckDownloadSchedule(void)
{
	short count=0;
	short ScheduleType = 0;
	struct _Node_	User_Node;
    NODE_ID		pNode;
	NODE_ID		p;
	char buf[MAX_PATH_LENGTH]="";
	char buf1[MAX_PATH_LENGTH]="";
	char DateTime[24]="";
	char max_dl_num[8]="0";
//	char S_FileName[16]="";
	char Downloading = 0;
	char FoundFlag = 0;
	char DownloadDIR = 0;
	char BackupToPC = 0;
	FILE *fp1;
	
	pNode = (NODE_ID)(char *)(&User_Node);
	
	Init_Log_List();
    Read_File();
    p = LOG_LIST->List_Head;
    if ( p == NULL )
    {
    	printf("empty log\n");
    	return;
    }
    	
    do
    {
	    //if ( p->wait_status == 1 )
	    if ( p->Status == 4 )
	    {
	    	FoundFlag = 1;
	    	memcpy(pNode , p , sizeof(struct _Node_ ));
//	    	strcpy(S_FileName , p->S_FileName );
//	    	pNode->wait_status = 0;
	    	
	    	ScheduleType = GetReScheduleCommand((char *)p , buf1 , &DownloadDIR );
	    	
	    	
	    	#ifdef SCHEDULE_DEBUG
	    	printf("schedule downlaod re-do.\n");
	    	#endif
	    	break;
		}//end if ( p->Status == 4 )
		
		if(p->Status == 3 )
    	{
    		Downloading = 1;
    	}
    	
		count++;
		if ( p->Next != NULL )
			p=p->Next;
		else
		{
			#ifdef SCHEDULE_DEBUG
			printf("No found need to wait schedule downlaod.\n");
			#endif
			break;
		}
	}while( count < MAX_LOG_LIST_NUM);
//	Write_File(0);  //jack20040408 mark
	Remove_Node();

//jack20040408+ change method to write log	   
	if ( FoundFlag == 1 )
	{
		FoundFlag = 0;
		
		if ( ScheduleType == 0 )
		{
			InitDownloadAT( pNode->S_FileName , buf1 , buf  , pNode->O_Directory , DownloadDIR );
		}
		else
		{
			//jack20041007+ .modify for backup to remote PC
			if ( (pNode->O_Directory[0] == '/') && (pNode->O_Directory[1] == '/') )	
			{
				BackupToPC = 1;
			}
			InitBackupAT( pNode->S_FileName , buf1 , buf , pNode->O_Directory , pNode->URL , BackupToPC );
			//jack20041007 end
		}
		
	    GetDateTime(DateTime); //jack20040408+ update re-schedule download time
	    SetSchedule( pNode->S_FileName ,buf , DateTime );
	    pNode->Work_ID =Get_at_work_ID();
		pNode->Status = 3;
		WriteUserNode( (char*)pNode, pNode->S_FileName );

	}	
//jack30040408 end
	
	//jack20040420+ 
	if( ( FoundFlag == 0 ) && ( Downloading == 0 ) )
	{
		fp1 = fopen("/tmp/max_dl_num","w+");
		if ( fp1 != NULL )
		{
			fputs(max_dl_num , fp1);
			fclose(fp1);
		}
	}
	//jack20040420 end
}

/*--------------------------------------------------------------
* ROUTINE NAME - CheckDownloadSafe
*---------------------------------------------------------------
* FUNCTION: 2004/03/17 add
*
* INPUT:	O_Path --> schedule download directory
*           FileName --> save schedule download filename
*             
* OUTPUT:   None
* RETURN:   0 : success
*          -1 : fail
* NOTE:
*-------------------------------------------------------------*/
int CheckDownloadSafe( char *O_Path , char *FileName , char *NewSubName , int Type )
{
	short count=0;
	short	status=0;
	short len,len1,len2;
	short Download_Num = 0;
	short UpdateNewSubSatus=0;
	short SubNameNum = 0;
	NODE_ID		p;
	FILE *fp;
	char buf[128];
//	char *sp;
	char *cp;
	char *FileName1;
	
	if ( ( fp = fopen(S_LOG_FILE_PATH,"r") ) == NULL )
	{
		return 0;
	}
	fclose(fp); //jack20041117
	
	//jack20040607 modify 
	len = strlen(O_Path);
	if ( O_Path[len-1] != '/' )
	{
	sprintf(buf , "%s/",O_Path );
	}
	else
	{
		strcpy(buf , O_Path );
	}
	//jack20040607 end
	
	Init_Log_List();
    Read_File();
    p = LOG_LIST->List_Head;
    
    do
    {
	    if ( p!= NULL )
	    {
		    
		    switch(Type)
		    {
		    	case 0:
			    	//if( ( p->Status == 0 ) || ( p->Status == 2 ) || ( p->Status == 3 ) || ( p->Status == 4 ))
			    	if( ( p->Status == 2 ) || ( p->Status == 3 ) || ( p->Status == 4 ))
				    {
					    //sp = strstr( p->URL , FileName );
					    //if ( sp != NULL )
					    
					    len1 = strlen(p->URL);
						if ( p->URL[len1-1] == '/' )
						{
							p->URL[len1-1] = 0x00;
							FileName1 = strrchr(p->URL,'/');
							p->URL[len1-1] = '/';
							FileName1++;
						}
						else
						{
					    	FileName1 = strrchr( p->URL , '/' );
					    	FileName1++;  //jack20040601 point to downlaod file name for check download file if exist
					    }
					    
						#ifdef SCHEDULE_DEBUG
						printf("Compare FileName = %s\n",FileName);
						printf("Compare FileName1 = %s\n",FileName1);
						#endif
					    if (!strcmp(FileName1 , FileName))
					    {
					    	#ifdef SCHEDULE_DEBUG
					    	printf("p->O_Directory = %s\n",p->O_Directory);
					    	printf("Save to Directory = %s\n",buf);
					    	#endif
					    	if (!strcmp(p->O_Directory , buf))
					    	{
					    		status = -1;
					    		len1 = strlen(p->SubName);
					    		len2 = strlen(p->SubName);
				
					    		if( ( len1 == 0 ) && ( len2 != 0 ))
					    		{
					    			strcpy(NewSubName , p->SubName);
					    		}
					    		else
					    		{
					    			if ( len2 > len1 )
					    			{
					    				strcpy( NewSubName , p->SubName);
					    			}
					    			else
					    			{
					    				if( (UpdateNewSubSatus = strcmp( p->SubName , NewSubName )) > 0 )
										{
											cp = p->SubName;
											cp++;
											SubNameNum = atoi(cp);
											SubNameNum++;
											sprintf(NewSubName ,".%d",SubNameNum);
										}
					    			}
					    		}
					    	}
					    }
					    Download_Num++;
					}
					break;
					
				case 1:
				    if( ( p->Status == 2 ) || ( p->Status == 3 ) || ( p->Status == 4 ))
				    {
					    //sp = strstr( p->URL , FileName );
					    //if ( sp != NULL )
						
						//jack20040607+
					    len1 = strlen(p->URL);
						if ( p->URL[len1-1] == '/' )
						{
							p->URL[len1-1] = 0x00;
							FileName1 = strrchr(p->URL,'/');
							p->URL[len1-1] = '/';
							FileName1++;
						}
						else
						{
					    	FileName1 = strrchr( p->URL , '/' );
					    	FileName1++;  //jack20040601 point to downlaod file name for check download file if exist
					    }
					    //jack20040607 end
					    
					    if (!strcmp(FileName1 , FileName))
					    {
	//				    	printf("p->O_Directory = %s\n",p->O_Directory);
	//				    	printf("O_Path = %s\n",O_Path );
							//jack20040607+
					    	len = strlen(p->O_Directory);
					    	if (p->O_Directory[len-1] == '/' )
					    	{
					    		p->O_Directory[len-1] = 0x00;
					    	}
					    	//jack20040607 end
					    	
					    	//jack20040407+ for download file the same name at different folder.
					    	if (!strcmp(p->O_Directory , O_Path))
					    	{
					    		len = strlen(p->SubName);		//jack20040318+
					    		if ( len == 0 )
					    		{
					    			status = -1;
					    		}
					    	}
					    }
					    Download_Num++;
					}
					break;
//jack20040608 mark				
//				case 2:
//					if( ( p->Status == -1 ) || ( p->Status == 0 ))
//					{
//						//sp = strstr( p->URL , FileName );
//					    //if ( sp != NULL )
//					    
//					    FileName1 = strrchr( p->URL , '/' );
//						if (!strcmp(FileName1 , FileName))
//					    {
//					    	if (!strcmp(p->O_Directory , O_Path))
//				    		{
//					    		status = -1;
//					    	}
//					    	
//					    }
//					}
//					break;
//jack20040608 end				
				//jack20040601+
				//for smbclient type
				case 3:
					if( ( p->Status == 3 ) || ( p->Status == 4 ))
					{
						
						#ifdef SCHEDULE_DEBUG
						printf("Compare p->O_Directory = %s\n",p->O_Directory);
						printf("Compare O_Path = %s\n",O_Path);
						#endif
							
						if (!strcmp(p->O_Directory , O_Path))
				    	{
				    		len1 = strlen(p->URL);
							if ( p->URL[len1-1] == '/' )
							{
								p->URL[len1-1] = 0x00;
								FileName1 = strrchr(p->URL,'/');
								p->URL[len1-1] = '/';
								FileName1++;
							}
							else
							{
						    	FileName1 = strrchr( p->URL , '/' );
						    	FileName1++;
						    }
				    		
				    		#ifdef SCHEDULE_DEBUG
							printf("Compare FileName = %s\n",FileName);
							printf("Compare FileName1 = %s\n",FileName1);
							#endif
				    		if (!strcmp(FileName1 , FileName))
					    	{
					    		status = -1;
					    	}
				    	}
					}	
					break;
				//jack20040601 end
				
				default:
					break;
			}//end switch(Type)
		}
		else
		{
			break;
		}
		
		count++;
		if ( p->Next != NULL )
			p=p->Next;
		else
			break;
	}while( count < MAX_LOG_LIST_NUM);
	Remove_Node();
	
	Total_Download_File_Num = Download_Num;
	
	#ifdef SCHEDULE_DEBUG
	printf("CheckDownloadSafe status =%d\n",status);
	#endif
	
	return status;
}

/*--------------------------------------------------------------
* ROUTINE NAME - GetInfo
*---------------------------------------------------------------
* FUNCTION: 2004/03/19 add
*
* INPUT:	S_FileName --> name of schedule download 
*           Output --> 
*           Type  0 : clear SubName
* 
* OUTPUT:   None
* RETURN:   0 : success
*          -1 : fail
* NOTE:
*-------------------------------------------------------------*/
//int GetSetInfo( char *S_FileName , char *Output , int Type )
//{
//	short count=0;
//	//jack20040408+ change method to write log
//	unsigned long offset = 0;  
//	char SubName[8];
//	//jack20040408 end
//	NODE_ID		p;
//	FILE *fp;
//	
//	if ( ( fp = fopen(S_LOG_FILE_PATH,"r") ) == NULL )
//	{
//		return -1;
//	}
//	
//	Init_Log_List();
//    Read_File();
//    p = LOG_LIST->List_Head;
//    
//    do
//    {
//	    if ( !strcmp( S_FileName , p->S_FileName ) )
//	    {
//	    	break;
//		}
//		
//		count++;
//		if ( p->Next != NULL )
//			p=p->Next;
//		else
//			break;
//	}while( count < MAX_LOG_LIST_NUM);
//	
//	Remove_Node();
//	switch(Type)
//	{
//		case 0:
//			strcpy( Output , p->SubName);
//			break;
//			
//		case 1:
//			//jack20040408+ change method to write log		
//			memset( SubName , 0 , 8 );
//			offset = sizeof(struct _Node_)*count + SUB_NAME_OFFSET;
//			Write_block( SubName , offset , 8 );
////			memset(p->SubName , 0 , 8 );
//			//jack20040408 end
//			break;
//		default:
//			break;
//	}
//	
//	return 0;
//}

/*--------------------------------------------------------------
* ROUTINE NAME - ClearUserNodeStruct
*---------------------------------------------------------------
* FUNCTION: 2004.05.08 add
*
* INPUT:	None
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void ClearUserNodeStruct(char *Info)
{
	memset(Info , 0 , sizeof( struct _Node_));
}

/*--------------------------------------------------------------
* ROUTINE NAME - ReadUserNode
*---------------------------------------------------------------
* FUNCTION: 2004.05.08 add
*
* INPUT:Info 		-->	User Node struct
*		S_FileName	--> Schedule name
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
short ReadUserNode(char *Info , char *S_FileName )
{
	NODE_ID pNode = ( struct _Node_ * )Info;
	short count =0;
	NODE_ID		p;
	FILE *fp;
	char SearchFlag = 0;
	
	if ( ( fp = fopen(S_LOG_FILE_PATH,"r") ) == NULL )
	{
		return -1;
	}
	fclose(fp); //jack20041117
	
	Init_Log_List();
    Read_File();
    p = LOG_LIST->List_Head;
    
    do
    {
	    if ( !strcmp( S_FileName , p->S_FileName ) )
	    {
	    	SearchFlag = 1;
	    	memcpy(pNode , p , sizeof(struct _Node_ ));
	    	break;
		}
		
		count++;
		if ( p->Next != NULL )
			p=p->Next;
		else
			break;
	}while( count < MAX_LOG_LIST_NUM);
	
	Remove_Node();
	
	if ( SearchFlag == 1 )
	{
		return 0;
	}
	else
	{
		return -1;
	}
}

/*--------------------------------------------------------------
* ROUTINE NAME - WriteUserNode
*---------------------------------------------------------------
* FUNCTION: 2004.05.08 add
*
* INPUT:Info 		-->	User Node struct
*		S_FileName	--> Schedule name
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void WriteUserNode(char *Info , char *S_FileName )
{
	short count=0;
	unsigned long offset = 0;
	NODE_ID		p;
	FILE *fp;
	
	if ( ( fp = fopen(S_LOG_FILE_PATH,"r") ) == NULL )
	{
		return;
	}
	fclose(fp);  //jack20041117
	
	Init_Log_List();
    Read_File();
    p = LOG_LIST->List_Head;
    
    do
    {
	    if ( !strcmp( S_FileName , p->S_FileName ) )
	    {
	    	break;
		}
		
		count++;
		if ( p->Next != NULL )
			p=p->Next;
		else
			break;
	}while( count < MAX_LOG_LIST_NUM);
	
	Remove_Node();
	
	offset = sizeof(struct _Node_)*count;
	
	Write_block( Info , offset , sizeof(struct _Node_ ));
}

/*--------------------------------------------------------------
* ROUTINE NAME - PrintUserNode
*---------------------------------------------------------------
* FUNCTION: 2004.05.08 add
*
* INPUT:	Info --> User Node struct
* OUTPUT:   None
* RETURN:   None
* NOTE:
*-------------------------------------------------------------*/
void PrintUserNode(char *Info)
{
	#ifdef SCHEDULE_DEBUG
	NODE_ID pNode = ( struct _Node_ * )Info;
	printf("LoginUser = %s\n",pNode->LoginUser);
    printf("URL = %s\n",pNode->URL);
    printf("O_Directory = %s\n",pNode->O_Directory);
    printf("DateTime = %s\n",pNode->DateTime);
    printf("S_FileName = %s\n",pNode->S_FileName);
    printf("SubName = %s\n",pNode->SubName );
    printf("work_id = %d\n",pNode->Work_ID);
    printf("Status = %d\n",pNode->Status);
    printf("Length = %ld\n",pNode->Length);
//    printf("wait_status = %d\n",pNode->wait_status);
    printf("CheckDownloadFlag = %d\n",pNode->CheckDownloadFlag);
    printf("PID = %d\n",pNode->PID);
    printf("UpdateTime = %s\n",pNode->UpdateTime);
    printf("UserPasswd = %s\n",pNode->UserPasswd);
//    printf("Folder = %s\n",pNode->Folder);
//    printf("SmbclientID = %d\n",pNode->SmbclientID);
    printf("Cycle = %d\n",pNode->Cycle);
    printf("BackupDifferent = %d\n",pNode->BackupDifferent);
	printf("-----------------------------------------\n");
	#endif
}

short Inc_Download_Num(void)
{
	char max_dl_num[8]="";
	FILE *fp;
	short status = -1;
	
	fp = fopen("/tmp/max_dl_num","r+");
	if ( fp != NULL )
	{
		fgets( max_dl_num , 8 , fp );
		if ( max_dl_num[0] < 0x33 )
		{
			status = 0;
			max_dl_num[0]++;
			fseek(fp , -1 , 1 );
			fputs( max_dl_num , fp );
			fclose(fp);
		}
	}
	
	return status;
}

short Dec_Download_Num(void)
{
	char max_dl_num[8]="";
	FILE *fp;
	short status = -1;
	
	fp = fopen("/tmp/max_dl_num","r+");
	if ( fp != NULL )
	{
		fgets( max_dl_num , 8 , fp );
		if ( max_dl_num[0] > 0x30 )
		{
			status = 0;
			max_dl_num[0]--;
			fseek(fp , -1 , 1 );
			fputs(max_dl_num , fp);
		}
		fclose(fp);
		
	}
	
	return status;
}

#ifdef LOG_DEBUG
void Debug_Print(char *buf)
{
	char debugbuf[80]="";
	sprintf(debugbuf , "echo %s >> /web/debug.log",buf );
	system(debugbuf);
}
#endif
